@@ -3,6 +3,7 @@
 #include <QMessageBox>
 #include <QPrintDialog>
 #include <QPrinter>
+#include <QPrinterInfo>
 #include <QPainter>
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
 #include <QStandardPaths>
@@ -18,6 +19,7 @@
 #include "searchdialog.h"
 #include "listdialog.h"
 #include "system.h"
+#include "optimizer.h"
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -28,6 +30,7 @@ MainWindow::MainWindow(QWidget *parent) :
     fileDialog(NULL),
     searchDialog(NULL),
     listDialog(NULL),
+    bandpassDialog(NULL),
     currentSpeakerNumber(0),
     currentTabIndex(-1),
     sealedPlot(NULL),
@@ -84,19 +87,16 @@ MainWindow::MainWindow(QWidget *parent) :
         /* index != 1 needs to deactivate the ported alignments menu */
         setActivateActions(ui->menuPorted_alignments->actions(), false);
     }
+    if (currentTabIndex != 2) {
+        /* index != 2 needs to deactivate the bandpass alignment menu */
+        ui->actionBandpass_alignement->setEnabled(false);
+    }
 
     syncUiFromCurrentSpeaker(currentSpeaker);
     syncUiFromCurrentSealedBox(currentSealedBox);
     syncUiFromCurrentPortedBox(currentPortedBox);
     syncUiFromCurrentBandPassBox(currentBandPassBox);
 
-    /* display speaker's values */
-    ui->fsValueLabel->setNum(currentSpeaker.getFs());
-    ui->qtsValueLabel->setNum(currentSpeaker.getQts());
-    ui->vasValueLabel->setNum(currentSpeaker.getVas());
-    ui->sdValueLabel->setNum(currentSpeaker.getSd());
-    ui->reValueLabel->setNum(currentSpeaker.getRe());
-
     /*
      * connections
      */
@@ -109,48 +109,13 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(ui->modelComboBox, SIGNAL(activated(QString)), this, SLOT(onModelChanged(QString)));
 
     /* menu actions */
-    connect(ui->actionProjectSave, SIGNAL(triggered()), this, SLOT(onProjectSave()));
-    connect(ui->actionProjectQuit, SIGNAL(triggered()), this, SLOT(onProjectQuit()));
-    connect(ui->actionSpeakerNew, SIGNAL(triggered()), this, SLOT(onSpeakerNew()));
-    connect(ui->actionSpeakerModify, SIGNAL(triggered()), this, SLOT(onSpeakerModify()));
-    connect(ui->actionSpeakerRemove, SIGNAL(triggered()), this, SLOT(onSpeakerRemove()));
-    connect(ui->actionProjectExport, SIGNAL(triggered()), this, SLOT(onProjectExport()));
-    connect(ui->actionProjectImport, SIGNAL(triggered()), this, SLOT(onProjectImport()));
-    connect(ui->actionEditOptimize, SIGNAL(triggered()), this, SLOT(onEditOptimize()));
-    connect(ui->actionSpeakerSearch, SIGNAL(triggered()), this, SLOT(onSpeakerSearch()));
-    connect(ui->actionProjectPrint, SIGNAL(triggered()), this, SLOT(onProjectPrint()));
-    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(onAboutAbout()));
-
-    /* ported alignments sub-menu */
-    connect(ui->actionModerate_Inf, SIGNAL(triggered()), this, SLOT(onAlignModerate_Inf()));
-    connect(ui->actionLegendre, SIGNAL(triggered()), this, SLOT(onAlignLegendre()));
-    connect(ui->actionBessel, SIGNAL(triggered()), this, SLOT(onAlignBessel()));
-    connect(ui->actionBullock, SIGNAL(triggered()), this, SLOT(onAlignBullock()));
-    connect(ui->actionKeele_Hoge, SIGNAL(triggered()), this, SLOT(onAlignKeele_Hoge()));
+    linkMenus();
 
     /* tabs actions */
-    connect(ui->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(onCurrentTabChanged(int)));
-
-    connect(ui->sealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onSealedVolumeDoubleSpinChanged(double)));
-
-    connect(ui->portedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedVolumeDoubleSpinChanged(double)));
-    connect(ui->portedResonancedoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedResonanceDoubleSpinChanged(double)));
-    connect(ui->portedPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onPortedPortsNumberSpinChanged(int)));
-    connect(ui->portedPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedPortDiameterDoubleSpinChanged(double)));
-    connect(ui->portedPortSlotWidthDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedSlotWidthDoubleSpinChanged(double)));
-
-    connect(ui->bandPassSealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassSealedVolumeDoubleSpinChanged(double)));
-    connect(ui->bandPassPortedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedVolumeDoubleSpinChanged(double)));
-    connect(ui->bandPassPortedResonanceDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedResonanceDoubleSpinChanged(double)));
-    connect(ui->bandPassPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onBandPassPortNumSpinChanged(int)));
-    connect(ui->bandPassPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortDiameterDoubleSpinChanged(double)));
-
-    /* app state */
-    connect(this, SIGNAL(currentSpeakerChanged(Speaker)), this, SLOT(onCurrentSpeakerChanged(const Speaker&)));
-    connect(this, SIGNAL(currentSealedBoxChanged(SealedBox)), this, SLOT(onCurrentSealedBoxChanged(const SealedBox&)));
-    connect(this, SIGNAL(currentPortedBoxChanged(PortedBox)), this, SLOT(onCurrentPortedBoxChanged(const PortedBox&)));
-    connect(this, SIGNAL(currentBandPassBoxChanged(BandPassBox)), this, SLOT(onCurrentBandPassBoxChanged(const BandPassBox&)));
+    linkTabs();
 
+    /* non-graphic */
+    linkInternals();
 }
 
 MainWindow::~MainWindow()
@@ -192,6 +157,9 @@ void MainWindow::onProjectImport()
     if (fileName.isEmpty())
         return;
 
+    unlinkInternals();
+    unlinkTabs();
+
     QFile file(fileName);
     ImportExport::importProject(currentSpeaker, currentSealedBox, currentPortedBox, currentBandPassBox, &currentSpeakerNumber, &currentTabIndex, file);
     ImportExport::setSavePath(fileName);
@@ -204,10 +172,9 @@ void MainWindow::onProjectImport()
     syncUiFromCurrentSealedBox(currentSealedBox);
     syncUiFromCurrentPortedBox(currentPortedBox);
     syncUiFromCurrentBandPassBox(currentBandPassBox);
-    emit currentSpeakerChanged(currentSpeaker);
-    emit currentSealedBoxChanged(currentSealedBox);
-    emit currentPortedBoxChanged(currentPortedBox);
-    emit currentBandPassBoxChanged(currentBandPassBox);
+
+    linkInternals();
+    linkTabs();
 }
 
 void MainWindow::onProjectQuit()
@@ -217,8 +184,10 @@ void MainWindow::onProjectQuit()
         return;
     }
 
-    if (!currentSpeaker.isValid())
+    if (!currentSpeaker.isValid()) {
         this->close();
+        return;
+    }
 
     QMessageBox::StandardButton pressed =
             QMessageBox::question(this, tr("Project not saved"),
@@ -269,6 +238,13 @@ void MainWindow::syncUiFromCurrentSpeaker(const Speaker& spk)
             }
         }
     }
+
+    /* display speaker's values */
+    ui->fsValueLabel->setNum(currentSpeaker.getFs());
+    ui->qtsValueLabel->setNum(currentSpeaker.getQts());
+    ui->vasValueLabel->setNum(currentSpeaker.getVas());
+    ui->sdValueLabel->setNum(currentSpeaker.getSd());
+    ui->reValueLabel->setNum(currentSpeaker.getRe());
 }
 
 void MainWindow::syncUiFromCurrentSealedBox(const SealedBox& box)
@@ -339,34 +315,23 @@ bool MainWindow::print(QPrinter *printer)
 
     QRect page = printer->pageRect();
 
-    qreal step = page.height() / 8;
-
-    currentSpeaker.render(&painter, QRectF(page.left(), page.top(), page.width(), step));
+    qreal step = page.height() / 4.0;
 
     if (ui->tabWidget->currentWidget() == ui->sealedTab) {
         System s(currentSpeaker, &currentSealedBox, currentSpeakerNumber);
-        s.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() / 16));
-        step += page.height() / 16;
-        currentSealedBox.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() / 16));
-        step += page.height() / 16;
+        s.render(&painter, QRectF(page.left(), page.top(), page.width(), step));
         sealedPlot->setUseOpenGL(false);
         sealedPlot->render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() - step));
         sealedPlot->setUseOpenGL(true);
     } else if (ui->tabWidget->currentWidget() == ui->portedTab) {
         System s(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
-        s.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() / 16));
-        step += page.height() / 16;
-        currentPortedBox.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() / 16));
-        step += page.height() / 16;
+        s.render(&painter, QRectF(page.left(), page.top(), page.width(), step));
         portedPlot->setUseOpenGL(false);
         portedPlot->render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() - step));
         portedPlot->setUseOpenGL(true);
     } else {
         System s(currentSpeaker, &currentBandPassBox, currentSpeakerNumber);
-        s.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height()/ 16));
-        step += page.height() / 16;
-        currentBandPassBox.render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() / 16));
-        step += page.height() / 16;
+        s.render(&painter, QRectF(page.left(), page.top(), page.width(), step));
         bandpassPlot->setUseOpenGL(false);
         bandpassPlot->render(&painter, QRectF(page.left(), page.top() + step, page.width(), page.height() - step));
         bandpassPlot->setUseOpenGL(true);
@@ -387,6 +352,142 @@ QString MainWindow::getHome()
     return home;
 }
 
+void MainWindow::onBandpassAlignment()
+{
+    bandpassDialog = new BandpassDialog(this);
+    connect(bandpassDialog, SIGNAL(optimizeRequested(double, double)), this, SLOT(onBandpassOptimizeRequested(double, double)));
+    connect(bandpassDialog, SIGNAL(optimizeCancelled()), this, SLOT(onBandpassOptimizeCancelled()));
+    bandpassDialog->show();
+}
+
+void MainWindow::onBandpassOptimizeRequested(double s, double pa)
+{
+    if (ui->tabWidget->currentWidget() == ui->bandpassTab) {
+        Optimizer optimizer(currentSpeaker, &currentBandPassBox, currentSpeakerNumber);
+        optimizer.bandpassAlignS_Pa(s, pa);
+        emit currentBandPassBoxChanged(currentBandPassBox);
+    }
+
+    disconnect(bandpassDialog, SIGNAL(optimizeRequested(double,double)), this, SLOT(onBandpassOptimizeRequested(double,double)));
+    disconnect(bandpassDialog, SIGNAL(optimizeCancelled()), this, SLOT(onBandpassOptimizeCancelled()));
+    bandpassDialog->close();
+    delete bandpassDialog;
+    bandpassDialog = NULL;
+}
+
+void MainWindow::onBandpassOptimizeCancelled()
+{
+    disconnect(bandpassDialog, SIGNAL(optimizeRequested(double,double)), this, SLOT(onBandpassOptimizeRequested(double,double)));
+    disconnect(bandpassDialog, SIGNAL(optimizeCancelled()), this, SLOT(onBandpassOptimizeCancelled()));
+    bandpassDialog->close();
+    delete bandpassDialog;
+    bandpassDialog = NULL;
+}
+
+void MainWindow::linkMenus()
+{
+    connect(ui->actionProjectSave, SIGNAL(triggered()), this, SLOT(onProjectSave()));
+    connect(ui->actionProjectQuit, SIGNAL(triggered()), this, SLOT(onProjectQuit()));
+    connect(ui->actionSpeakerNew, SIGNAL(triggered()), this, SLOT(onSpeakerNew()));
+    connect(ui->actionSpeakerModify, SIGNAL(triggered()), this, SLOT(onSpeakerModify()));
+    connect(ui->actionSpeakerRemove, SIGNAL(triggered()), this, SLOT(onSpeakerRemove()));
+    connect(ui->actionProjectExport, SIGNAL(triggered()), this, SLOT(onProjectExport()));
+    connect(ui->actionProjectImport, SIGNAL(triggered()), this, SLOT(onProjectImport()));
+    connect(ui->actionEditOptimize, SIGNAL(triggered()), this, SLOT(onEditOptimize()));
+    connect(ui->actionSpeakerSearch, SIGNAL(triggered()), this, SLOT(onSpeakerSearch()));
+    connect(ui->actionProjectPrint, SIGNAL(triggered()), this, SLOT(onProjectPrint()));
+    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(onAboutAbout()));
+
+    /* ported alignments sub-menu */
+    connect(ui->actionModerate_Inf, SIGNAL(triggered()), this, SLOT(onAlignModerate_Inf()));
+    connect(ui->actionLegendre, SIGNAL(triggered()), this, SLOT(onAlignLegendre()));
+    connect(ui->actionBessel, SIGNAL(triggered()), this, SLOT(onAlignBessel()));
+    connect(ui->actionBullock, SIGNAL(triggered()), this, SLOT(onAlignBullock()));
+    connect(ui->actionKeele_Hoge, SIGNAL(triggered()), this, SLOT(onAlignKeele_Hoge()));
+
+    /* bandpass alignment */
+    connect(ui->actionBandpass_alignement, SIGNAL(triggered(bool)), this, SLOT(onBandpassAlignment()));
+}
+
+void MainWindow::linkTabs()
+{
+    connect(ui->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(onCurrentTabChanged(int)));
+
+    connect(ui->sealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onSealedVolumeDoubleSpinChanged(double)));
+
+    connect(ui->portedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedVolumeDoubleSpinChanged(double)));
+    connect(ui->portedResonancedoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedResonanceDoubleSpinChanged(double)));
+    connect(ui->portedPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onPortedPortsNumberSpinChanged(int)));
+    connect(ui->portedPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedPortDiameterDoubleSpinChanged(double)));
+    connect(ui->portedPortSlotWidthDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedSlotWidthDoubleSpinChanged(double)));
+
+    connect(ui->bandPassSealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassSealedVolumeDoubleSpinChanged(double)));
+    connect(ui->bandPassPortedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedVolumeDoubleSpinChanged(double)));
+    connect(ui->bandPassPortedResonanceDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedResonanceDoubleSpinChanged(double)));
+    connect(ui->bandPassPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onBandPassPortNumSpinChanged(int)));
+    connect(ui->bandPassPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortDiameterDoubleSpinChanged(double)));
+}
+
+void MainWindow::linkInternals()
+{
+    connect(this, SIGNAL(currentSpeakerChanged(Speaker)), this, SLOT(onCurrentSpeakerChanged(const Speaker&)));
+    connect(this, SIGNAL(currentSealedBoxChanged(SealedBox)), this, SLOT(onCurrentSealedBoxChanged(const SealedBox&)));
+    connect(this, SIGNAL(currentPortedBoxChanged(PortedBox)), this, SLOT(onCurrentPortedBoxChanged(const PortedBox&)));
+    connect(this, SIGNAL(currentBandPassBoxChanged(BandPassBox)), this, SLOT(onCurrentBandPassBoxChanged(const BandPassBox&)));
+}
+
+void MainWindow::unlinkMenus()
+{
+    disconnect(ui->actionProjectSave, SIGNAL(triggered()), this, SLOT(onProjectSave()));
+    disconnect(ui->actionProjectQuit, SIGNAL(triggered()), this, SLOT(onProjectQuit()));
+    disconnect(ui->actionSpeakerNew, SIGNAL(triggered()), this, SLOT(onSpeakerNew()));
+    disconnect(ui->actionSpeakerModify, SIGNAL(triggered()), this, SLOT(onSpeakerModify()));
+    disconnect(ui->actionSpeakerRemove, SIGNAL(triggered()), this, SLOT(onSpeakerRemove()));
+    disconnect(ui->actionProjectExport, SIGNAL(triggered()), this, SLOT(onProjectExport()));
+    disconnect(ui->actionProjectImport, SIGNAL(triggered()), this, SLOT(onProjectImport()));
+    disconnect(ui->actionEditOptimize, SIGNAL(triggered()), this, SLOT(onEditOptimize()));
+    disconnect(ui->actionSpeakerSearch, SIGNAL(triggered()), this, SLOT(onSpeakerSearch()));
+    disconnect(ui->actionProjectPrint, SIGNAL(triggered()), this, SLOT(onProjectPrint()));
+    disconnect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(onAboutAbout()));
+
+    /* ported alignments sub-menu */
+    disconnect(ui->actionModerate_Inf, SIGNAL(triggered()), this, SLOT(onAlignModerate_Inf()));
+    disconnect(ui->actionLegendre, SIGNAL(triggered()), this, SLOT(onAlignLegendre()));
+    disconnect(ui->actionBessel, SIGNAL(triggered()), this, SLOT(onAlignBessel()));
+    disconnect(ui->actionBullock, SIGNAL(triggered()), this, SLOT(onAlignBullock()));
+    disconnect(ui->actionKeele_Hoge, SIGNAL(triggered()), this, SLOT(onAlignKeele_Hoge()));
+
+    /* bandpass alignment */
+    disconnect(ui->actionBandpass_alignement, SIGNAL(triggered(bool)), this, SLOT(onBandpassAlignment()));
+}
+
+void MainWindow::unlinkTabs()
+{
+    disconnect(ui->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(onCurrentTabChanged(int)));
+
+    disconnect(ui->sealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onSealedVolumeDoubleSpinChanged(double)));
+
+    disconnect(ui->portedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedVolumeDoubleSpinChanged(double)));
+    disconnect(ui->portedResonancedoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedResonanceDoubleSpinChanged(double)));
+    disconnect(ui->portedPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onPortedPortsNumberSpinChanged(int)));
+    disconnect(ui->portedPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedPortDiameterDoubleSpinChanged(double)));
+    disconnect(ui->portedPortSlotWidthDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onPortedSlotWidthDoubleSpinChanged(double)));
+
+    disconnect(ui->bandPassSealedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassSealedVolumeDoubleSpinChanged(double)));
+    disconnect(ui->bandPassPortedVolumeDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedVolumeDoubleSpinChanged(double)));
+    disconnect(ui->bandPassPortedResonanceDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortedResonanceDoubleSpinChanged(double)));
+    disconnect(ui->bandPassPortsNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onBandPassPortNumSpinChanged(int)));
+    disconnect(ui->bandPassPortDiameterDoubleSpinBox, SIGNAL(valueChanged(double)), this, SLOT(onBandPassPortDiameterDoubleSpinChanged(double)));
+}
+
+void MainWindow::unlinkInternals()
+{
+    disconnect(this, SIGNAL(currentSpeakerChanged(Speaker)), this, SLOT(onCurrentSpeakerChanged(const Speaker&)));
+    disconnect(this, SIGNAL(currentSealedBoxChanged(SealedBox)), this, SLOT(onCurrentSealedBoxChanged(const SealedBox&)));
+    disconnect(this, SIGNAL(currentPortedBoxChanged(PortedBox)), this, SLOT(onCurrentPortedBoxChanged(const PortedBox&)));
+    disconnect(this, SIGNAL(currentBandPassBoxChanged(BandPassBox)), this, SLOT(onCurrentBandPassBoxChanged(const BandPassBox&)));
+}
+
 void MainWindow::onSpeakerNew()
 {
     isModifying = false;
@@ -467,32 +568,16 @@ void MainWindow::onSpeakerCancelled()
 void MainWindow::onEditOptimize()
 {
     if (ui->tabWidget->currentWidget() == ui->sealedTab) {
-        double qr = 0.707 / currentSpeaker.getQts();
-        double vr = pow(qr, 2.0) - 1;
-        currentSealedBox.setVolume(currentSpeaker.getVas() * currentSpeakerNumber / vr);
+        Optimizer optimizer(currentSpeaker, &currentSealedBox, currentSpeakerNumber);
+        optimizer.genericOptimizeBox();
         emit currentSealedBoxChanged(currentSealedBox);
     } else if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        /* approx. "natural max flat", a good starting point */
-        currentPortedBox.setBoxVolume(20 * currentSpeaker.getVas() * currentSpeakerNumber * pow(currentSpeaker.getQts(), 3.3));
-        currentPortedBox.setResFreq(currentSpeaker.getFs() * pow(currentSpeaker.getVas() * currentSpeakerNumber / currentPortedBox.getBoxVolume(), 0.31));
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimizer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimizer.genericOptimizeBox();
         emit currentPortedBoxChanged(currentPortedBox);
     } else {
-        /* optimization constants */
-        double pa = 0.0;
-        double s = 0.7;
-        //double b = 0.7206;
-
-        double qbp = pow((pow(10.0,(-pa/40.0)) * 2 * s ), -1);
-        //double fl  = (-b + pow(pow(b, 2.0) + 4 * pow(qbp, 2.0), 0.5) / 2.0) * (currentSpeaker.getFs() / currentSpeaker.getQts());
-        //double fh  = fl + (b * currentSpeaker.getFs() / currentSpeaker.getQts());
-        double fb  = qbp * currentSpeaker.getFs() / currentSpeaker.getQts();
-        double vf  = pow(2 * s * currentSpeaker.getQts(), 2) * currentSpeaker.getVas() * currentSpeakerNumber;
-        double vr  = currentSpeaker.getVas() * currentSpeakerNumber / (pow(qbp / currentSpeaker.getQts(), 2) - 1);
-        currentBandPassBox.setSealedBoxVolume(vr);
-        currentBandPassBox.setPortedBoxVolume(vf);
-        currentBandPassBox.setPortedBoxResFreq(fb);
-        currentBandPassBox.updatePortedBoxPorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimizer(currentSpeaker, &currentBandPassBox, currentSpeakerNumber);
+        optimizer.genericOptimizeBox();
         emit currentBandPassBoxChanged(currentBandPassBox);
     }
 }
@@ -500,10 +585,8 @@ void MainWindow::onEditOptimize()
 void MainWindow::onAlignModerate_Inf()
 {
     if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        /* use M4 Moderate alignment (see http://www.mzbinden.ch/ventedalignments/index.html) */
-        currentPortedBox.setBoxVolume((2.52 * currentSpeaker.getQts() - 0.35) * currentSpeaker.getVas() * currentSpeakerNumber);
-        currentPortedBox.setResFreq(0.32 * sqrt((1.0/pow(currentSpeaker.getQts(), 2.0)) + 3.38) * currentSpeaker.getFs());
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimzer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimzer.portedAlignModerate_Inf();
         emit currentPortedBoxChanged(currentPortedBox);
     }
 }
@@ -511,9 +594,8 @@ void MainWindow::onAlignModerate_Inf()
 void MainWindow::onAlignLegendre()
 {
     if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        currentPortedBox.setBoxVolume(10.728 * pow(currentSpeaker.getQts(), 2.4186) * currentSpeaker.getVas() * currentSpeakerNumber);
-        currentPortedBox.setResFreq(0.3802 * pow(currentSpeaker.getQts(), -1.0657) * currentSpeaker.getFs());
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimizer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimizer.portedAlignLegendre();
         emit currentPortedBoxChanged(currentPortedBox);
     }
 }
@@ -521,9 +603,8 @@ void MainWindow::onAlignLegendre()
 void MainWindow::onAlignBessel()
 {
     if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        currentPortedBox.setBoxVolume(8.0707 * pow(currentSpeaker.getQts(), 2.5848) * currentSpeaker.getVas() * currentSpeakerNumber);
-        currentPortedBox.setResFreq(0.3552 * pow(currentSpeaker.getQts(), -0.9649) * currentSpeaker.getFs());
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimzer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimzer.portedAlignBessel();
         emit currentPortedBoxChanged(currentPortedBox);
     }
 }
@@ -532,9 +613,8 @@ void MainWindow::onAlignKeele_Hoge()
 {
     /* BB4 */
     if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        currentPortedBox.setBoxVolume(currentSpeaker.getVas() * currentSpeakerNumber * 5.2358 * pow(currentSpeaker.getQts(), 2.1687));
-        currentPortedBox.setResFreq(currentSpeaker.getFs());
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimizer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimizer.portedAlignKeele_Hoge();
         emit currentPortedBoxChanged(currentPortedBox);
     }
 }
@@ -543,10 +623,8 @@ void MainWindow::onAlignBullock()
 {
     /* SBB4 */
     if (ui->tabWidget->currentWidget() == ui->portedTab) {
-        double qts = currentSpeaker.getQts();
-        currentPortedBox.setBoxVolume(qts * currentSpeaker.getVas() * currentSpeakerNumber * (4.96 * qts - 0.136));
-        currentPortedBox.setResFreq(currentSpeaker.getFs());
-        currentPortedBox.updatePorts(currentSpeaker.getSd() * currentSpeakerNumber, currentSpeaker.getXmax());
+        Optimizer optimizer(currentSpeaker, &currentPortedBox, currentSpeakerNumber);
+        optimizer.portedAlignBullock();
         emit currentPortedBoxChanged(currentPortedBox);
     }
 }
@@ -563,10 +641,19 @@ void MainWindow::onProjectPrint()
 {
     QPrinter printer;
     printer.setCreator("QSpeakers");
+    printer.setDocName("qspeakers_project");
     printer.setOrientation(QPrinter::Landscape);
     QPrintDialog dialog(&printer, this);
     if (dialog.exec() == QDialog::Accepted) {
-       print(&printer);
+        if (printer.isValid()) {
+            if (!print(&printer)) {
+                QPrinterInfo info(printer);
+                qWarning() << "printerinfo definition null:" << info.isNull();
+                qWarning() << "printerinfo state error:" << (info.state() == QPrinter::Error);
+            }
+        } else {
+            qWarning() << "invalid printer object";
+        }
     }
 }
 
@@ -647,18 +734,24 @@ void MainWindow::onCurrentTabChanged(int tab)
     case 0:
     {
         setActivateActions(alignments, false);
+        ui->actionBandpass_alignement->setEnabled(false);
+
         syncUiFromCurrentSealedBox(currentSealedBox);
         break;
     }
     case 1:
     {
         setActivateActions(alignments, true);
+        ui->actionBandpass_alignement->setEnabled(false);
+
         syncUiFromCurrentPortedBox(currentPortedBox);
         break;
     }
     case 2:
     {
         setActivateActions(alignments, false);
+        ui->actionBandpass_alignement->setEnabled(true);
+
         syncUiFromCurrentBandPassBox(currentBandPassBox);
         break;
     }
@@ -680,12 +773,9 @@ void MainWindow::onNumberSpinChanged(int number) {
     }
 
     projectSaved = false;
-    if (ui->tabWidget->currentWidget() == ui->sealedTab)
-        syncUiFromCurrentSealedBox(currentSealedBox);
-    else if (ui->tabWidget->currentWidget() == ui->portedTab)
-        syncUiFromCurrentPortedBox(currentPortedBox);
-    else
-        syncUiFromCurrentBandPassBox(currentBandPassBox);
+    syncUiFromCurrentSealedBox(currentSealedBox);
+    syncUiFromCurrentPortedBox(currentPortedBox);
+    syncUiFromCurrentBandPassBox(currentBandPassBox);
 }
 
 void MainWindow::onVendorChanged(QString vendor)
@@ -751,12 +841,9 @@ void MainWindow::onCurrentSpeakerChanged(const Speaker &spk)
     }
 
     projectSaved = false;
-    if (ui->tabWidget->currentWidget() == ui->sealedTab)
-        syncUiFromCurrentSealedBox(currentSealedBox);
-    else if (ui->tabWidget->currentWidget() == ui->portedTab)
-        syncUiFromCurrentPortedBox(currentPortedBox);
-    else
-        syncUiFromCurrentBandPassBox(currentBandPassBox);
+    syncUiFromCurrentSealedBox(currentSealedBox);
+    syncUiFromCurrentPortedBox(currentPortedBox);
+    syncUiFromCurrentBandPassBox(currentBandPassBox);
 }
 
 void MainWindow::onCurrentSealedBoxChanged(const SealedBox &box)
@@ -765,8 +852,7 @@ void MainWindow::onCurrentSealedBoxChanged(const SealedBox &box)
         return;
 
     projectSaved = false;
-    if (ui->tabWidget->currentWidget() == ui->sealedTab)
-        syncUiFromCurrentSealedBox(box);
+    syncUiFromCurrentSealedBox(box);
 }
 
 void MainWindow::onCurrentPortedBoxChanged(const PortedBox &box)
@@ -775,8 +861,7 @@ void MainWindow::onCurrentPortedBoxChanged(const PortedBox &box)
         return;
 
     projectSaved = false;
-    if (ui->tabWidget->currentWidget() == ui->portedTab)
-        syncUiFromCurrentPortedBox(box);
+    syncUiFromCurrentPortedBox(box);
 }
 
 void MainWindow::onCurrentBandPassBoxChanged(const BandPassBox &box)
@@ -785,8 +870,7 @@ void MainWindow::onCurrentBandPassBoxChanged(const BandPassBox &box)
         return;
 
     projectSaved = false;
-    if (ui->tabWidget->currentWidget() == ui->bandpassTab)
-        syncUiFromCurrentBandPassBox(box);
+    syncUiFromCurrentBandPassBox(box);
 }
 
 void MainWindow::onSealedVolumeDoubleSpinChanged(double val)
