@@ -41,7 +41,7 @@ void SealedBox::fromDomElement(const QDomElement &e)
     volume = c.toDouble(e.attribute("volume"));
 }
 
-void SealedBox::render(QPainter *painter, const QRectF &area)
+void SealedBox::render(QPainter *painter, const QRectF &area) const
 {    
     if (!painter)
         return;
